#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import socket

from .tls import TLSModifier
from ..util_core.v2ray import restart
from ..util_core.selector import GroupSelector
from ..util_core.writer import StreamWriter, GroupWriter
from ..util_core.utils import StreamType, ColorStr, get_ip, loop_input_choice_number, check_ip, is_ipv4

# https://support.cloudflare.com/hc/en-us/articles/200169156-Identifying-network-ports-compatible-with-Cloudflare-s-proxy
class CDNModifier:
    def __init__(self, group_tag='A', group_index=-1, domain=''):
        self.domain = domain
        self.group_tag = group_tag
        self.group_index = group_index
        if domain:
            StreamWriter(self.group_tag, self.group_index, StreamType.WS).write()

        self.gw = GroupWriter(group_tag, group_index)
    
    @restart()
    def openHttp(self, port=80):
        '''
        cloudflare cdn proxy http port(80, 8080, 8880, 2052, 2082, 2086, 2095)
        '''
        self.gw.write_port(port)
        self.gw.write_domain(self.domain)
        return True

    def openHttps(self, port=443):
        '''
        cloudflare cdn proxy https port(443, 2053, 2083, 2087, 2096, 8443)
        '''
        self.gw.write_port(port)
        TLSModifier(self.group_tag, self.group_index, self.domain).turn_on()
    
    @restart()
    def closeHttp(self):
        self.gw.write_domain()
        return True

def modify():
    choice, port_choice = "", ""
    gs = GroupSelector(_("modify cdn"))
    group = gs.group

    http_list=(80, 8080, 8880, 2052, 2082, 2086, 2095)
    https_list=(443, 2053, 2083, 2087, 2096, 8443)

    if group == None:
        pass
    else:
        print("")
        print(_("1.open http cdn"))
        print(_("2.open https cdn"))
        print(_("3.close http cdn"))
        choice = loop_input_choice_number(_("please select: "), 3)
        if not choice:
            return

        if choice == 3:
            if group.port not in list(map(str, http_list)):
                print(ColorStr.yellow(_("only support http port cdn close!")))
                return
            CDNModifier(group.tag, group.index).closeHttp()
            return

        if check_ip(group.ip):
            domain = input(_("please input run cdn mode domain: "))
            if not domain:
                print(ColorStr.yellow(_("domain is empty!")))
                return
        else:
            domain = group.ip

        local_ip = get_ip()
        try:
            if is_ipv4(local_ip):
                input_ip = socket.gethostbyname(domain)
            else:
                input_ip = socket.getaddrinfo(domain, None, socket.AF_INET6)[0][4][0]
        except Exception:
            print(_("domain check error!!!"))
            print("")
            return
        
        print("")
        if choice == 1:
            for index, text in enumerate(http_list): 
                print("{}.{}".format(index + 1, text))
            port_choice = loop_input_choice_number(_("please select http port to cdn: "), len(http_list))
            if not port_choice:
                return
            CDNModifier(group.tag, group.index, domain).openHttp(http_list[port_choice - 1])

        elif choice == 2:
            print(_("local vps ip address: ") + local_ip + "\n")

            if input_ip != local_ip:
                print(_("domain can't analysis to local ip!!!"))
                print(_("must be close cdn proxy!"))
                print("")
                return

            for index, text in enumerate(https_list): 
                print("{}.{}".format(index + 1, text))
            port_choice = loop_input_choice_number(_("please select https port to cdn: "), len(https_list))
            if not port_choice:
                return
            print("")
            CDNModifier(group.tag, group.index, domain).openHttps(https_list[port_choice - 1])


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import gettext
import pkg_resources

lang = 'en'
if os.path.exists('/etc/v2ray_util/util.cfg'):
    from .config import Config
    lang = Config().get_data('lang')
if lang == 'zh':
    trans = gettext.translation('lang', pkg_resources.resource_filename('v2ray_util', 'locale_i18n'), languages=['zh_CH'])
else:
    trans = gettext.translation('lang', pkg_resources.resource_filename('v2ray_util', 'locale_i18n'), languages=['en_US'])
trans.install()
_ = trans.gettext


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import random
import sys

from ..util_core.v2ray import restart
from ..util_core.writer import NodeWriter, GroupWriter
from ..util_core.group import Vmess, Socks, Mtproto, SS
from ..util_core.selector import GroupSelector, ClientSelector
from ..util_core.utils import StreamType, stream_list, is_email, clean_iptables, random_email, ColorStr, readchar 

@restart(True)
def new_port(new_stream=None):
    info = dict()
    if new_stream:
        correct_list = stream_list()
        if new_stream not in [x.value for x in correct_list]:
            print(_("input error! input -h or --help to get help"))
            exit(-1)
        
        stream = list(filter(lambda stream:stream.value == new_stream, correct_list))[0]

        if stream == StreamType.SOCKS:
            user = input(_("please input socks user: "))
            password = input(_("please input socks password: "))
            if user == "" or password == "":
                print(_("socks user or password is null!!"))
                exit(-1)
            info = {"user":user, "pass": password}
        elif stream == StreamType.SS:
            from .ss import SSFactory
            sf = SSFactory()
            info = {"method": sf.get_method(), "password": sf.get_password()}
    else:
        salt_stream = [StreamType.KCP_DTLS, StreamType.KCP_WECHAT, StreamType.KCP_UTP, StreamType.KCP_SRTP, StreamType.KCP_WG]
        random.shuffle(salt_stream)
        stream = salt_stream[0]
        print("{}: {} \n".format(_("random generate (srtp | wechat-video | utp | dtls | wireguard) fake header, new protocol"), ColorStr.green(stream.value)))

    random_port = random.randint(1000, 65535)
    new_port = input("{0} {1}, {2}: ".format(_("random generate port"), ColorStr.green(str(random_port)), _("enter to use, or input customize port")))

    if not new_port:
        new_port = str(random_port)

    if new_port.isnumeric():
        print("")
        print("{}: {}".format(_("new port"), new_port))
        print("")
        nw = NodeWriter()
        nw.create_new_port(int(new_port), stream, **info)
        return True
    else:
        print(_("input error, please check is number"))

@restart()
def new_user():
    gs = GroupSelector(_('add user'))
    group = gs.group
    group_list = gs.group_list

    if group == None:
        pass
    else:
        email = ""
        if type(group.node_list[0]) == Vmess: 
            while True:
                is_duplicate_email=False
                remail = random_email()
                tip = _("create random email:") + ColorStr.cyan(remail) + _(", enter to use it or input new email: ")
                email = input(tip)
                if email == "":
                    email = remail
                if not is_email(email):
                    print(_("not email, please input again"))
                    continue
                
                for loop_group in group_list:
                    for node in loop_group.node_list:
                        if node.user_info == None or node.user_info == '':
                            continue
                        elif node.user_info == email:
                            print(_("have same email, please input other"))
                            is_duplicate_email = True
                            break              
                if not is_duplicate_email:
                    break

            nw = NodeWriter(group.tag, group.index)
            info = {'email': email}
            nw.create_new_user(**info)
            return True

        elif type(group.node_list[0]) == Socks:
            print(_("local group is socks, please input user and password to create user"))
            print("")
            user = input(_("please input socks user: "))
            password = input(_("please input socks password: "))
            if user == "" or password == "":
                print(_("socks user or password is null!!"))
                exit(-1)
            info = {"user":user, "pass": password}
            nw = NodeWriter(group.tag, group.index)
            nw.create_new_user(**info)
            return True

        elif type(group.node_list[0]) == Mtproto:
            print("")
            print(_("Mtproto protocol only support one user!!"))

        elif type(group.node_list[0]) == SS:
            print("")
            print(_("Shadowsocks protocol only support one user, u can add new port to multiple SS!"))

@restart()
def del_port():
    gs = GroupSelector(_('del port'))
    group = gs.group

    if group == None:
        pass
    else:
        print(_("del group info: "))
        print(group)
        choice = readchar(_("delete?(y/n): ")).lower()
        if choice == 'y':
            nw = NodeWriter()
            nw.del_port(group)
            clean_iptables(group.port)
            return True
        else:
            print(_("undo delete"))

@restart()
def del_user():
    cs = ClientSelector(_('del user'))
    group = cs.group

    if group == None:
        pass
    else:
        client_index = cs.client_index
        print(_("del user info:"))
        print(group.show_node(client_index))
        choice = readchar(_("delete?(y/n): ")).lower()
        if choice == 'y':
            if len(group.node_list) == 1:
                clean_iptables(group.port)
            nw = NodeWriter()
            nw.del_user(group, client_index)
            return True
        else:
            print(_("undo delete"))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import platform
import subprocess
import pkg_resources

from ..util_core.loader import Loader
from ..util_core.utils import ColorStr, calcul_iptables_traffic, readchar

def manage():
    if "centos-8" in platform.platform():
        print("Centos8 don't support iptables traffic statistics")
        return
    
    loader = Loader()

    profile = loader.profile

    group_list = profile.group_list

    while True:
        print("")
        print(_("Iptables Traffic Statistics"))
        print("")
        print(_("1.check statistics result"))
        print("")
        print(_("2.reset special port statistics"))
        print("")

        choice = readchar(_("please select: "))
        if choice == "1":
            print("")
            ipv6 = True if profile.network == "ipv6" else False
            for group in group_list:
                print(calcul_iptables_traffic(group.port, ipv6))
            print("")

        elif choice == "2":
            port = input(_("please input reset port:"))
            if port and port.isnumeric():
                subprocess.call("bash {0} {1}".format(pkg_resources.resource_filename(__name__, "clean_traffic.sh"), str(port)), shell=True)
                print(ColorStr.green(_("reset success!")))
            else:
                print(ColorStr.red(_("input error!")))
        else:
            break


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import uuid
import time
import random
import subprocess
import pkg_resources
from functools import wraps
from .utils import ColorStr, open_port, get_ip, is_ipv6

def restart(port_open=False):
    """
    运行函数/方法后重启v2ray的装饰器
    """  
    def decorate(func):
        @wraps(func)
        def wrapper(*args, **kw):
            result = func(*args, **kw)
            if port_open:
                open_port()
            if result:
                V2ray.restart()
        return wrapper
    return decorate

class V2ray:

    @staticmethod
    def docker_run(command, keyword):
        subprocess.run(command, shell=True)
        print("{}ing v2ray...".format(keyword))
        time.sleep(1)
        if V2ray.docker_status() or keyword == "stop":
            print(ColorStr.green("v2ray {} success !".format(keyword)))
        else:
            print(ColorStr.red("v2ray {} fail !".format(keyword)))

    @staticmethod
    def run(command, keyword):
        try:
            subprocess.check_output(command, shell=True)
            print("{}ing v2ray...".format(keyword))
            time.sleep(2)
            if subprocess.check_output("systemctl is-active v2ray|grep active", shell=True) or keyword == "stop":
                print(ColorStr.green("v2ray {} success !".format(keyword)))
            else:
                raise subprocess.CalledProcessError
        except subprocess.CalledProcessError:
            print(ColorStr.red("v2ray {} fail !".format(keyword)))

    @staticmethod
    def docker_status():
        is_running = True
        failed = bytes.decode(subprocess.run('cat /.run.log|grep failed', shell=True, stdout=subprocess.PIPE).stdout)
        running = bytes.decode(subprocess.run('ps aux|grep /etc/v2ray/config.json', shell=True, stdout=subprocess.PIPE).stdout)
        if failed or "/usr/bin/v2ray/v2ray" not in running:
            is_running = False
        return is_running

    @staticmethod
    def status():
        if os.path.exists("/.dockerenv"):
            if V2ray.docker_status():
                print(ColorStr.green("v2ray running.."))
            else:
                print(bytes.decode(subprocess.run('cat /.run.log', shell=True, stdout=subprocess.PIPE).stdout))
                print(ColorStr.yellow("v2ray stoped.."))
        else:
            subprocess.call("systemctl status v2ray", shell=True)

    @staticmethod
    def version():
        v2ray_version = bytes.decode(subprocess.check_output("/usr/bin/v2ray/v2ray -version | head -n 1 | awk '{print $2}'", shell=True))
        import v2ray_util
        print("v2ray: {}".format(ColorStr.green(v2ray_version)))
        print("v2ray_util: {}".format(ColorStr.green(v2ray_util.__version__)))   

    @staticmethod
    def info():
        from .loader import Loader 
        print(Loader().profile)

    @staticmethod
    def update():
        if is_ipv6(get_ip()):
            print(ColorStr.yellow(_("ipv6 network not support update v2ray online, please manual donwload v2ray to update!")))
            print(ColorStr.fuchsia(_("download v2ray-linux-xx.zip and run 'bash <(curl -L -s https://install.direct/go.sh) -l v2ray-linux-xx.zip' to update")))
            return
        subprocess.Popen("curl -L -s https://install.direct/go.sh|bash", shell=True).wait()

    @staticmethod
    def cleanLog():
        subprocess.call("cat /dev/null > /var/log/v2ray/access.log", shell=True)
        subprocess.call("cat /dev/null > /var/log/v2ray/error.log", shell=True)
        print(ColorStr.green(_("clean v2ray log success!")))
        print("")

    @staticmethod
    def log():
        f = subprocess.Popen(['tail','-f', '-n', '100', '/var/log/v2ray/access.log'],
                stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        while True:
            print(bytes.decode(f.stdout.readline().strip()))

    @classmethod
    def restart(cls):
        if os.path.exists("/.dockerenv"):
            V2ray.stop()
            V2ray.start()
        else:
            cls.run("systemctl restart v2ray", "restart")

    @classmethod
    def start(cls):
        if os.path.exists("/.dockerenv"):
            cls.docker_run("/usr/bin/v2ray/v2ray -config=/etc/v2ray/config.json > /.run.log &", "start")
        else:
            cls.run("systemctl start v2ray", "start")

    @classmethod
    def stop(cls):
        if os.path.exists("/.dockerenv"):
            cls.docker_run('''ps aux|grep "/usr/bin/v2ray/v2ray"|awk '{print $1}'|xargs  -r kill -9 2>/dev/null''', "stop")
        else:
            cls.run("systemctl stop v2ray", "stop")

    @classmethod
    def convert(cls):
        from .converter import ConfigConverter

    @classmethod
    def check(cls):
        if not os.path.exists("/etc/v2ray_util/util.cfg"):
            subprocess.call("mkdir -p /etc/v2ray_util && cp -f {} /etc/v2ray_util/".format(pkg_resources.resource_filename(__name__, 'util.cfg')), shell=True)
        if not os.path.exists("/usr/bin/v2ray/v2ray"):
            print(ColorStr.yellow(_("check v2ray no install, auto install v2ray..")))
            if is_ipv6(get_ip()):
                subprocess.Popen("curl -Ls https://install.direct/go.sh -o temp.sh", shell=True).wait()
                subprocess.Popen("bash temp.sh --source jsdelivr && rm -f temp.sh", shell=True).wait()
            else:
                cls.update()
            cls.new()

    @classmethod
    def new(cls):
        subprocess.call("rm -rf /etc/v2ray/config.json && cp {}/server.json /etc/v2ray/config.json".format(pkg_resources.resource_filename('v2ray_util', "json_template")), shell=True)
        new_uuid = uuid.uuid1()
        print("new UUID: {}".format(ColorStr.green(str(new_uuid))))
        random_port = random.randint(1000, 65535)
        print("new port: {}".format(ColorStr.green(str(random_port))))
        subprocess.call("sed -i \"s/cc4f8d5b-967b-4557-a4b6-bde92965bc27/{0}/g\" /etc/v2ray/config.json && sed -i \"s/999999999/{1}/g\" /etc/v2ray/config.json".format(new_uuid, random_port), shell=True)
        from ..config_modify import stream
        stream.StreamModifier().random_kcp()
        open_port()
        cls.restart()